#!/usr/bin/env python3
# encoding: utf-8

import re
import sys
from datetime import datetime

VERSION_FILE = "CURRENT_VERSION.txt"
CHANGELOG_FILE = "CHANGES.txt"

def is_valid_version(version):
    """ Check if a string is a valid version
    """
    return bool(re.search("[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}", version))

def validate(version):
    """ Check if a version is valid, thro error if not
    """
    if is_valid_version(version):
        return version
    else:
        msg = "{} is not a valid version".format(version)
        raise ValueError(msg)

def get_current_version():
    """ Get the current version from file
    """
    with open(VERSION_FILE) as f:
        current_version = f.read().strip()
        validate(current_version)
        return current_version

def _increment_version(version):
    """ Increment version 0.0.1 => 0.0.2
    """
    validate(version)
    parts = version.split(".")
    last_digit = int(parts[-1])
    last_digit += 1
    incremented_version = ".".join(parts[:-1] + [str(last_digit)])
    return incremented_version

def main():
    """
    current_version = get_current_version()
    suggested_version = _increment_version(current_version)

    # Ask user for version
    new_version = raw_input('New version (currently {}) [{}]: '\
        .format(current_version, suggested_version))
    new_version = new_version or suggested_version
    validate(new_version)

    # Ask user for message
    msg = raw_input("Describe version changes (will be added to {}): "\
        .format(CHANGELOG_FILE))
    """
    if len(sys.argv) < 3:
        raise ValueError("Must provide version and message as arguments.")


    new_version = sys.argv[1]
    validate(new_version)
    msg = sys.argv[2]

    date = datetime.now().strftime("%Y-%m-%d")

    # Update changelog
    # Format: v<version>, <date> -- Initial release.
    change_log_row = "v{}, {} -- {}".format(new_version, date, msg)
    with open(CHANGELOG_FILE, 'a') as file:
        file.write(change_log_row + "\n")
        print(u"{} updated: '{}'".format(CHANGELOG_FILE, change_log_row))

    # Update CURRENT_VERSION
    with open(VERSION_FILE, 'r+') as file:
        file.write(new_version)


if __name__ == '__main__':
    main()



"""Functions for fetching parametes from SSM.
"""
import os
from boto3 import client as boto3_client
from botocore.exceptions import ClientError, NoCredentialsError, NoRegionError
from aws_utils.exceptions import SsmError

def env_or_ssm(env_key=None, ssm_key=None, required=True, aws_region=None,
               aws_access_key_id=None, aws_secret_access_key=None):
    """Try to get an environmet variable first from env, then AWS SSM.
    Raises error if required is True.
    """
    value = None

    # 1. try to get from env
    if env_key is not None:
        value = os.environ.get(env_key)

    # 2. try to get from SSM
    if value is None or value == "":
        try:
            value = get_ssm_parameter(ssm_key, aws_region=aws_region,
                                      aws_access_key_id=aws_access_key_id,
                                      aws_secret_access_key=aws_secret_access_key)
        except NoCredentialsError:
            pass

    # 3. Error if required
    if (value is None or value == "") and required:
        raise SsmError("Required setting undefined. Set {} in environment or "
                       "or {} in AWS SSM.".format(env_key, ssm_key))

    return value


def get_ssm_parameter(key, aws_region=None, aws_access_key_id=None,
                      aws_secret_access_key=None):
    """Fetch value of an AWS SSM parameter.

    `aws_region`,  `aws_access_key_id` and `aws_secret_access_key` may be
    defined if you haven't set them up in your environment or if you want to
    override the environment.
    Read more about setting up credentials here:
    https://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html#configuring-credentials
    """
    client_params = {}
    if aws_region is not None:
        client_params["region_name"] = aws_region
    if aws_access_key_id is not None:
        client_params["aws_access_key_id"] = aws_access_key_id
    if aws_secret_access_key is not None:
        client_params["aws_secret_access_key"] = aws_secret_access_key

    try:
        ssm_client = boto3_client('ssm', **client_params)
        resp = ssm_client.get_parameter(Name=key, WithDecryption=True)
        val = resp['Parameter']['Value']
    except (ClientError, NoRegionError) as e:
        val = None
    return val



# encoding: utf-8
import pytest
from aws_utils import env_or_ssm
from aws_utils.exceptions import SsmError

def test_env_or_ssm():
    """This test must be run with access to Newsworthy SSM params
    """
    param = env_or_ssm("EXTERNAL_DATA_JWT_TOKEN", "EXTERNAL_DATA_JWT_TOKEN")
    assert param is not None

def test_env_or_ssm_non_existing():
    with pytest.raises(SsmError):
        param = env_or_ssm("NO_EXISTING", "NO_EXISTING")

    param = env_or_ssm("NO_EXISTING", "NO_EXISTING", required=False)
    assert param is None

def test_env_or_ssm_with_region():
    param = env_or_ssm("EXTERNAL_DATA_JWT_TOKEN", "EXTERNAL_DATA_JWT_TOKEN",
                       aws_region="eu-central-1")
    assert param is not None



#!/usr/bin/env python3
from subprocess import call
from new_version import get_current_version

v = get_current_version()
call(["python3", "setup.py", "sdist", "bdist_wheel"])
call(["python3", "-m", "twine", "upload", "dist/nw_aws_utils-{}*".format(v)])
